import { DomManipulation } from '@rxweb/dom';
import { MultiLingualData } from '@rxweb/localization';

class BaseDomProvider {
    constructor() {
        this.elements = {};
    }
    createElement(parentElement, elementName, elementConfig, modelObject, index, customEvents) {
        var domManipulation = new DomManipulation(parentElement, elementName, elementConfig, modelObject, index, customEvents);
        domManipulation.bind();
        this.elements[domManipulation.controlId] = domManipulation;
        if (elementConfig.sourceItems && elementConfig.childrens)
            elementConfig.sourceItems.forEach((t, index) => {
                var childrenLength = elementConfig.childrens.length;
                this.createChildElements(domManipulation.element, [childrenLength > index ? elementConfig.childrens[index] : elementConfig.childrens[0]], t, index, customEvents);
            });
        else if (elementConfig.childrens)
            this.createChildElements(domManipulation.element, elementConfig.childrens, modelObject, 0, customEvents);
        return domManipulation;
    }
    createChildElements(element, childrens, modelObject, index, customEvents) {
        childrens.forEach(templateConfig => {
            Object.keys(templateConfig).forEach(t => {
                var result = true;
                if (templateConfig[t].if)
                    result = templateConfig[t].if.call(modelObject);
                if (templateConfig[t].event && templateConfig[t].event.onclick)
                    if (customEvents[templateConfig[t].event.onclick])
                        templateConfig[t].event.onclick = customEvents[templateConfig[t].event.onclick];
                if (result)
                    this.createElement(element, t, templateConfig[t], modelObject, index, customEvents);
            });
        });
    }
    removeChildren(element, isRemoveRoot = true) {
        if (element) {
            while (element.firstElementChild && element.firstElementChild.tagName.indexOf("-") === -1)
                this.removeChildren(element.firstElementChild);
            if (isRemoveRoot) {
                let controlId = element.getAttribute("data-rxwebid");
                if (controlId && this.elements[controlId]) {
                    this.elements[controlId].destroy();
                    delete this.elements[controlId];
                }
            }
        }
    }
}

var DialogClick;
(function (DialogClick) {
    DialogClick[DialogClick["Ok"] = 1] = "Ok";
    DialogClick[DialogClick["Cancel"] = 2] = "Cancel";
    DialogClick[DialogClick["Save"] = 3] = "Save";
    DialogClick[DialogClick["DontSave"] = 4] = "DontSave";
})(DialogClick || (DialogClick = {}));

class DialogClass {
    constructor() {
        this.root = [];
        this.nestedDiv = [];
        this.heading = [];
        this.paragraph = [];
    }
}
class DialogButtonClass {
    constructor() {
        this.ok = [];
        this.cancel = [];
        this.save = [];
        this.dontSave = [];
    }
}

class OverlayDesign {
    constructor() {
        this.class = [];
        this.style = {};
    }
}

var DialogViewMode;
(function (DialogViewMode) {
    DialogViewMode[DialogViewMode["onlyOk"] = 1] = "onlyOk";
    DialogViewMode[DialogViewMode["okWithCancel"] = 2] = "okWithCancel";
    DialogViewMode[DialogViewMode["saveAndDontSave"] = 3] = "saveAndDontSave";
})(DialogViewMode || (DialogViewMode = {}));

function getDefaultTemplate(dialogClass) {
    return {
        div: {
            class: dialogClass.root,
            style: dialogClass.rootStyle,
            childrens: [{
                    h2: {
                        class: dialogClass.heading, childrens: [{ text: { text: ":text.heading" } }]
                    },
                    p: { class: dialogClass.paragraph, childrens: [{ text: { text: ":text.message" } }] }
                },
                {
                    div: {
                        class: dialogClass.nestedDiv,
                        childrens: [{
                                button: {
                                    if: function () { return this.viewMode == DialogViewMode.onlyOk || this.viewMode == DialogViewMode.okWithCancel; },
                                    class: dialogClass.button.ok,
                                    event: { click: 'ok' },
                                    childrens: [{ text: { text: ":text.ok" } }]
                                }
                            },
                            {
                                button: {
                                    if: function () { return this.viewMode == DialogViewMode.saveAndDontSave; },
                                    event: { click: 'save' },
                                    class: dialogClass.button.save, childrens: [{ text: { text: ":text.save" } }]
                                }
                            },
                            {
                                button: {
                                    if: function () { return this.viewMode == DialogViewMode.saveAndDontSave; },
                                    event: { click: 'dontsave' },
                                    class: dialogClass.button.dontSave, childrens: [{ text: { text: ":text.dontsave" } }]
                                }
                            },
                            {
                                button: {
                                    if: function () { return this.viewMode == DialogViewMode.okWithCancel || this.viewMode == DialogViewMode.saveAndDontSave; },
                                    event: { click: 'cancel' },
                                    class: dialogClass.button.cancel, childrens: [{ text: { text: ":text.cancel" } }]
                                }
                            },
                        ]
                    }
                }
            ]
        }
    };
}

class DialogConfig {
    constructor(message) {
        this.showIcon = false;
        this.text = new DialogTextConfig(message);
    }
}
class DialogTextConfig {
    constructor(message) {
        this.ok = MultiLingualData.get("global.dialog-ok");
        this.cancel = MultiLingualData.get("global.dialog-cancel");
        this.save = MultiLingualData.get("global.dialog-save");
        this.dontSave = MultiLingualData.get("global.dialog-dontsave");
        this.heading = MultiLingualData.get("global.dialog-heading");
        this.message = message.charAt(0) == ":" ? MultiLingualData.get(message.replace(":", "")) : message;
    }
}

class Dialog extends BaseDomProvider {
    constructor() {
        super();
        this.designClass = new DialogClass();
        this.designClass.button = new DialogButtonClass();
        this.defaultConfig = new DialogConfig("global.dialog-message");
        this.designClass.overlay = new OverlayDesign();
    }
    getDefaultTemplate() {
        return getDefaultTemplate(this.designClass);
    }
    show(message, viewMode, template) {
        let promise = new Promise((resolve, reject) => {
            var events = this.getEvents(resolve);
            var template = template || this.getDefaultTemplate();
            var data;
            if (message instanceof DialogConfig)
                data = Object.assign({}, message, { viewMode: viewMode });
            else
                data = Object.assign({}, this.defaultConfig, { text: Object.assign({}, this.defaultConfig.text, { message: message }) }, { viewMode: viewMode });
            if (this.designClass.overlay.class || this.designClass.overlay.style)
                this.overLayElement = this.createElement(document.body, 'div', { class: this.designClass.overlay.class, style: this.designClass.overlay.style }, {}, 0, {});
            this.domManipulation = this.createElement(document.body, 'div', template.div, data, 0, { actions: events });
        });
        return promise;
    }
    getEvents(resolve) {
        return { ok: () => resolve(DialogClick.Ok), cancel: () => resolve(DialogClick.Cancel), save: () => resolve(DialogClick.Save), dontsave: () => resolve(DialogClick.DontSave) };
    }
    hide() {
        this.removeChildren(this.overLayElement.element);
        this.removeChildren(this.domManipulation.element);
    }
}

class ModalClassConfig {
    constructor() {
        this.class = [];
        this.showIn = [];
        this.showOut = [];
        this.backDropClass = [];
        this.overlay = new OverlayDesign();
    }
}

class Modal extends BaseDomProvider {
    constructor() {
        super();
        this.designClass = new ModalClassConfig();
    }
    getDefaultTemplate() {
        return {
            div: {
                class: this.designClass.class,
                style: this.designClass.style
            }
        };
    }
    show(component, params) {
        let promise = new Promise((resolve, reject) => {
            if (this.resolver) {
                var template = this.getDefaultTemplate();
                let overLayElement = this.createElement(document.body, "div", { class: this.designClass.backDropClass }, {}, 0, {});
                let domManipulation = this.createElement(document.body, "div", template.div, {}, 0, {});
                let refResolver = {};
                let componentRef = this.resolver(component, Object.assign({}, params, { hide: this.hideResolver(resolve, domManipulation, overLayElement, refResolver) }));
                refResolver.destroy = componentRef.destroy;
                domManipulation.element.appendChild(componentRef.element);
                if (this.designClass.showIn.length > 0)
                    setTimeout(() => {
                        domManipulation.addOrRemoveClass(this.designClass.showIn);
                    }, 50);
            }
        });
        return promise;
    }
    hideResolver(resolve, domManipulation, overLayElement, refResolver) {
        return (data) => {
            this.hide(resolve, data, domManipulation, overLayElement, refResolver);
        };
    }
    hide(resolve, data, domManipulation, overLayElement, refResolver) {
        if (this.designClass.showIn)
            domManipulation.addOrRemoveClass(this.designClass.showIn, false);
        setTimeout(() => {
            if (refResolver.destroy)
                refResolver.destroy();
            this.removeChildren(overLayElement.element);
            this.removeChildren(domManipulation.element);
            resolve(data);
        }, 200);
    }
}

class ToastrConfig {
    constructor() {
        this.timeOut = 7000;
    }
}
class ToastrDesignClass {
    constructor() {
        this.root = [];
        this.secondLevelDiv = [];
        this.thirdLevelDiv = [];
        this.showClass = [];
        this.hideClass = [];
    }
}

function getToastrTemplate(toastrDesign, config, hideConfig) {
    return {
        div: {
            class: toastrDesign.root,
            childrens: [{
                    div: {
                        class: toastrDesign.secondLevelDiv,
                        childrens: [{
                                div: {
                                    class: toastrDesign.thirdLevelDiv,
                                    childrens: [
                                        {
                                            button: {
                                                childrens: [{ text: { text: "x" } }],
                                                class: ["toast-close-button", config && (config.autoHideDisable || config.enableCloseButton) ? "dummy" : "toast-close-display-none"],
                                                event: {
                                                    click: (event) => {
                                                        if (config && (config.autoHideDisable || config.enableCloseButton) && hideConfig.domManipulation) {
                                                            hideConfig.hideFunc(hideConfig.domManipulation, hideConfig.toastrConfig);
                                                        }
                                                    }
                                                }
                                            }
                                        },
                                        {
                                            text: { text: ":message" }
                                        }
                                    ]
                                }
                            }]
                    }
                }]
        }
    };
}

class Toastr extends BaseDomProvider {
    constructor() {
        super();
        this.designClass = new ToastrDesignClass();
        this.defaultConfig = new ToastrConfig();
        this.domManipulations = new Array();
    }
    getDefaultTemplate(config, hideConfig) {
        return getToastrTemplate(this.designClass, config, hideConfig);
    }
    getMessage(message) {
        return message.charAt(0) == ':' ? MultiLingualData.get(message.replace(":", "")) : message;
    }
    show(message, messageType, config, customTemplate) {
        message = this.getMessage(message);
        let hideConfig = { toastrConfig: toastrConfig, hideFunc: this.baseHide.bind(this) };
        var toastrConfig = config ? Object.assign({}, config, { messageType: messageType, message: message, timeOut: this.defaultConfig.timeOut }) : Object.assign({}, this.defaultConfig, { messageType: messageType, message: message });
        var template = customTemplate || this.getDefaultTemplate(config, hideConfig);
        this.createRoot();
        var domManipulation = this.createElement(this.rootElement.element, 'div', template.div, toastrConfig, 0, {});
        hideConfig.domManipulation = domManipulation;
        hideConfig.toastrConfig = toastrConfig;
        if (this.designClass.showClass)
            domManipulation.addOrRemoveClass(this.designClass.showClass);
        if (!config || (config && (!config.autoHideDisable && config.enableCloseButton)) || (config && (!config.autoHideDisable && !config.enableCloseButton)))
            this.hide(domManipulation, toastrConfig);
        this.domManipulations.push(domManipulation);
    }
    createRoot() {
        if (!this.rootElement)
            this.rootElement = this.createElement(document.body, "div", {}, {}, 0, {});
    }
    hide(domManipulation, toastrConfig) {
        setTimeout(() => {
            this.baseHide(domManipulation, toastrConfig);
        }, toastrConfig.timeOut);
    }
    baseHide(domManipulation, toastrConfig) {
        if (this.designClass.hideClass) {
            domManipulation.addOrRemoveClass(this.designClass.showClass, false);
            domManipulation.addOrRemoveClass(this.designClass.hideClass);
        }
        setTimeout(() => {
            if (this.rootElement) {
                this.removeChildren(domManipulation.element);
                var elements = this.domManipulations.filter(t => t.element != null);
                if (elements.length == 0) {
                    this.domManipulations = [];
                    this.removeChildren(this.rootElement.element);
                    this.rootElement = null;
                }
            }
        }, 50);
    }
}

var ToastrMessageType;
(function (ToastrMessageType) {
    ToastrMessageType[ToastrMessageType["success"] = 1] = "success";
    ToastrMessageType[ToastrMessageType["error"] = 2] = "error";
    ToastrMessageType[ToastrMessageType["warning"] = 3] = "warning";
    ToastrMessageType[ToastrMessageType["info"] = 4] = "info";
})(ToastrMessageType || (ToastrMessageType = {}));

export { Dialog, DialogClick, DialogViewMode, Modal, Toastr, ToastrMessageType, BaseDomProvider as ɵa };
//# sourceMappingURL=rxweb-js.js.map
